@@ -4,10 +4,9 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '1.5019';
+our $VERSION = '1.5020';
 
 use Carp              ();
-use Fcntl             ();
 use File::Spec        ();
 use Exporter          ();
 use Data::MessagePack ();
@@ -377,7 +376,6 @@ sub slurp_template {
 
     open my($source), '<' . $input_layer, $fullpath
         or $self->_error("LoadError: Cannot open $fullpath for reading: $!");
-    flock($source, Fcntl::LOCK_SH());
     local $/;
     return scalar <$source>;
 }
@@ -410,21 +408,35 @@ sub _load_source {
         if(not -e $cachedir) {
             require File::Path;
             eval { File::Path::mkpath($cachedir) }
-                or Carp::croak("Xslate: Cannot prepare the cache directory $cachepath (ignored): $@");
+                or Carp::croak("Xslate: Cannot prepare cache directory $cachepath (ignored): $@");
         }
 
-        if(sysopen my($out), $cachepath, Fcntl::O_WRONLY() | Fcntl::O_CREAT()) {
-            flock $out, Fcntl::LOCK_EX();
-            truncate $out, 0 or Carp::croak("Xslate: failed to truncate: $!");
-            binmode($out);
-            $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
+        my $tmpfile = sprintf('%s.%d.d', $cachepath, $$, $self);
+
+        if (open my($out), ">:raw", $tmpfile) {
+            my $mtime = $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
 
             if(!close $out) {
                  Carp::carp("Xslate: Cannot close $cachepath (ignored): $!");
-                 unlink $cachepath;
+                 unlink $tmpfile;
+            }
+            elsif (rename($tmpfile => $cachepath)) {
+                # set the newest mtime of all the related files to cache mtime
+                if (not ref $fullpath) {
+                    my $main_mtime = (stat $fullpath)[_ST_MTIME];
+                    if (defined($main_mtime) && $main_mtime > $mtime) {
+                        $mtime = $main_mtime;
+                    }
+                    utime $mtime, $mtime, $cachepath;
+                    $fi->{cache_mtime} = $mtime;
+                }
+                else {
+                    $fi->{cache_mtime} = (stat $cachepath)[_ST_MTIME];
+                }
             }
             else {
-                $fi->{cache_mtime} = ( stat $cachepath )[_ST_MTIME];
+                Carp::carp("Xslate: Cannot rename cache file $cachepath (ignored): $!");
+                unlink $tmpfile;
             }
         }
         else {
@@ -463,7 +475,6 @@ sub _load_compiled {
     my $cachepath = $fi->{cachepath};
     open my($in), '<:raw', $cachepath
         or $self->_error("LoadError: Cannot open $cachepath for reading: $!");
-    flock($in, Fcntl::LOCK_SH());
 
     my $magic = $self->_magic_token($fi->{fullpath});
     my $data;
@@ -526,10 +537,19 @@ sub _save_compiled {
     local $\;
     print $out $self->_magic_token($fullpath);
     print $out $mp->pack($is_utf8 ? 1 : 0);
+
+    my $newest_mtime = 0;
     foreach my $c(@{$asm}) {
         print $out $mp->pack($c);
+
+        if ($c->[0] eq 'depend') {
+            my $dep_mtime = (stat $c->[1])[_ST_MTIME];
+            if ($newest_mtime < $dep_mtime) {
+                $newest_mtime = $dep_mtime;
+            }
+        }
     }
-    return;
+    return $newest_mtime;
 }
 
 sub _magic_token {
@@ -635,7 +655,7 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 1.5019.
+This document describes Text::Xslate version 1.5020.
 
 =head1 SYNOPSIS
 
